@@ -128,15 +128,10 @@
 
 #define READ_CHUNK_SIZE 8192
 #define RECURSE_LEVEL_MAX 4
-#define DIR_MIME_TYPE "x-directory/normal"
-#define BLOCK_DEVICE_TYPE "x-special/device-block"
-#define EMPTY_FILE_TYPE "application/x-zerosize"
-#define TEXT_URI_TYPE "text/uri-list"
-#define AUDIO_MPEG_TYPE "audio/mpeg"
 
 #define D(x) if (debug) x
 
-typedef gboolean (*PlaylistIdenCallback) (const char *data, gsize len);
+typedef const char * (*PlaylistIdenCallback) (const char *data, gsize len);
 
 #ifndef TOTEM_PL_PARSER_MINI
 typedef TotemPlParserResult (*PlaylistCallback) (TotemPlParser *parser, const char *url, const char *base, gpointer data);
@@ -153,6 +148,66 @@ typedef struct {
 #endif
 } PlaylistTypes;
 
+#ifndef TOTEM_PL_PARSER_MINI
+#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime, cb, identcb, unsafe }
+#define PLAYLIST_TYPE2(mime,cb,identcb) { mime, cb, identcb }
+#define PLAYLIST_TYPE3(mime) { mime, NULL, NULL, FALSE }
+#else
+#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime }
+#define PLAYLIST_TYPE2(mime,cb,identcb) { mime, identcb }
+#define PLAYLIST_TYPE3(mime) { mime }
+#endif
+
+/* These ones need a special treatment, mostly parser formats */
+static PlaylistTypes special_types[] = {
+	PLAYLIST_TYPE ("audio/x-mpegurl", totem_pl_parser_add_m3u, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/playlist", totem_pl_parser_add_m3u, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-scpls", totem_pl_parser_add_pls, NULL, FALSE),
+	PLAYLIST_TYPE ("application/x-smil", totem_pl_parser_add_smil, NULL, FALSE),
+	PLAYLIST_TYPE ("application/smil", totem_pl_parser_add_smil, NULL, FALSE),
+	PLAYLIST_TYPE ("video/x-ms-wvx", totem_pl_parser_add_asx, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-ms-wax", totem_pl_parser_add_asx, NULL, FALSE),
+	PLAYLIST_TYPE ("application/xspf+xml", totem_pl_parser_add_xspf, NULL, FALSE),
+	PLAYLIST_TYPE ("text/uri-list", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list, FALSE),
+	PLAYLIST_TYPE ("text/x-google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
+	PLAYLIST_TYPE ("text/google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
+	PLAYLIST_TYPE ("audio/x-iriver-pla", totem_pl_parser_add_pla, NULL, FALSE),
+	PLAYLIST_TYPE ("application/atom+xml", totem_pl_parser_add_atom, NULL, FALSE),
+	PLAYLIST_TYPE ("application/rss+xml", totem_pl_parser_add_rss, totem_pl_parser_is_rss, FALSE),
+	PLAYLIST_TYPE ("text/x-opml+xml", totem_pl_parser_add_opml, NULL, FALSE),
+#ifndef TOTEM_PL_PARSER_MINI
+	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-cd-image", totem_pl_parser_add_iso, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-extension-img", totem_pl_parser_add_iso, NULL, TRUE),
+	PLAYLIST_TYPE ("application/x-cue", totem_pl_parser_add_cue, NULL, TRUE),
+	PLAYLIST_TYPE (DIR_MIME_TYPE, totem_pl_parser_add_directory, NULL, TRUE),
+	PLAYLIST_TYPE (BLOCK_DEVICE_TYPE, totem_pl_parser_add_block, NULL, TRUE),
+#endif
+};
+
+/* These ones are "dual" types, might be a video, might be a parser
+ * Please keep the same _is_ functions together */
+static PlaylistTypes dual_types[] = {
+	PLAYLIST_TYPE2 ("audio/x-real-audio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("audio/x-pn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("application/ram", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("application/vnd.rn-realmedia", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("audio/x-pn-realaudio-plugin", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("audio/vnd.rn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("audio/x-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("text/plain", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("audio/x-ms-asx", totem_pl_parser_add_asx, totem_pl_parser_is_asx),
+	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
+	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
+	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
+	PLAYLIST_TYPE2 ("application/xml", totem_pl_parser_add_xml_feed, totem_pl_parser_is_xml_feed),
+};
+
+static char *my_gnome_vfs_get_mime_type_for_data (gconstpointer data, int len);
+
 #ifndef TOTEM_PL_PARSER_MINI
 
 static void totem_pl_parser_set_property (GObject *object,
@@ -359,6 +414,10 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "Genre of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("album", "album",
+				     "Album of the item to be added", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("base", "base",
 				     "Base URL of the item to be added", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
@@ -372,7 +431,11 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("duration", "duration",
-				     "String representing the duration of the entry, used for still images", NULL,
+				     "String representing the duration of the entry", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("duration-ms", "duration-ms",
+				     "String representing the duration of the entry in milliseconds", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 	pspec = g_param_spec_string ("starttime", "starttime",
@@ -431,6 +494,14 @@ totem_pl_parser_class_init (TotemPlParserClass *klass)
 				     "String representing the location of an image for a playlist", NULL,
 				     G_PARAM_READABLE & G_PARAM_WRITABLE);
 	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("download-url", "download-url",
+				     "String representing the location of a download URL", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
+	pspec = g_param_spec_string ("id", "id",
+				     "String representing the identifier for an entry", NULL,
+				     G_PARAM_READABLE & G_PARAM_WRITABLE);
+	g_param_spec_pool_insert (totem_pl_parser_pspec_pool, pspec, TOTEM_TYPE_PL_PARSER);
 }
 
 static void
@@ -562,7 +633,6 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 	GnomeVFSResult result;
 	GnomeVFSHandle *handle;
 	char *buffer;
-	const char *mimetype;
 	GnomeVFSFileSize total_bytes_read;
 	GnomeVFSFileSize bytes_read;
 
@@ -639,14 +709,7 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 	buffer[total_bytes_read] = '\0';
 	*data = buffer;
 
-	mimetype = gnome_vfs_get_mime_type_for_data (*data, total_bytes_read);
-
-	if (mimetype != NULL && strcmp (mimetype, "text/plain") == 0) {
-		if (totem_pl_parser_is_uri_list (*data, total_bytes_read) != FALSE)
-			return g_strdup (TEXT_URI_TYPE);
-	}
-
-	return g_strdup (mimetype);
+	return my_gnome_vfs_get_mime_type_for_data (*data, total_bytes_read);
 }
 
 /**
@@ -1299,67 +1362,6 @@ totem_pl_parser_resolve_url (const char *base, const char *url)
 	return resolved;
 }
 
-#endif /* !TOTEM_PL_PARSER_MINI */
-
-#ifndef TOTEM_PL_PARSER_MINI
-#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime, cb, identcb, unsafe }
-#define PLAYLIST_TYPE2(mime,cb,identcb) { mime, cb, identcb }
-#define PLAYLIST_TYPE3(mime) { mime, NULL, NULL, FALSE }
-#else
-#define PLAYLIST_TYPE(mime,cb,identcb,unsafe) { mime }
-#define PLAYLIST_TYPE2(mime,cb,identcb) { mime, identcb }
-#define PLAYLIST_TYPE3(mime) { mime }
-#endif
-
-/* These ones need a special treatment, mostly parser formats */
-static PlaylistTypes special_types[] = {
-	PLAYLIST_TYPE ("audio/x-mpegurl", totem_pl_parser_add_m3u, NULL, FALSE),
-	PLAYLIST_TYPE ("audio/playlist", totem_pl_parser_add_m3u, NULL, FALSE),
-	PLAYLIST_TYPE ("audio/x-scpls", totem_pl_parser_add_pls, NULL, FALSE),
-	PLAYLIST_TYPE ("application/x-smil", totem_pl_parser_add_smil, NULL, FALSE),
-	PLAYLIST_TYPE ("application/smil", totem_pl_parser_add_smil, NULL, FALSE),
-	PLAYLIST_TYPE ("video/x-ms-wvx", totem_pl_parser_add_asx, NULL, FALSE),
-	PLAYLIST_TYPE ("audio/x-ms-wax", totem_pl_parser_add_asx, NULL, FALSE),
-	PLAYLIST_TYPE ("application/xspf+xml", totem_pl_parser_add_xspf, NULL, FALSE),
-	PLAYLIST_TYPE ("text/uri-list", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list, FALSE),
-	PLAYLIST_TYPE ("text/x-google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
-	PLAYLIST_TYPE ("text/google-video-pointer", totem_pl_parser_add_gvp, NULL, FALSE),
-	PLAYLIST_TYPE ("audio/x-iriver-pla", totem_pl_parser_add_pla, NULL, FALSE),
-	PLAYLIST_TYPE ("application/atom+xml", totem_pl_parser_add_atom, NULL, FALSE),
-	PLAYLIST_TYPE ("application/rss+xml", totem_pl_parser_add_rss, NULL, FALSE),
-	PLAYLIST_TYPE ("text/x-opml+xml", totem_pl_parser_add_opml, NULL, FALSE),
-#ifndef TOTEM_PL_PARSER_MINI
-	PLAYLIST_TYPE ("application/x-desktop", totem_pl_parser_add_desktop, NULL, TRUE),
-	PLAYLIST_TYPE ("application/x-gnome-app-info", totem_pl_parser_add_desktop, NULL, TRUE),
-	PLAYLIST_TYPE ("application/x-cd-image", totem_pl_parser_add_iso, NULL, TRUE),
-	PLAYLIST_TYPE ("application/x-extension-img", totem_pl_parser_add_iso, NULL, TRUE),
-	PLAYLIST_TYPE ("application/x-cue", totem_pl_parser_add_cue, NULL, TRUE),
-	PLAYLIST_TYPE (DIR_MIME_TYPE, totem_pl_parser_add_directory, NULL, TRUE),
-	PLAYLIST_TYPE (BLOCK_DEVICE_TYPE, totem_pl_parser_add_block, NULL, TRUE),
-#endif
-};
-
-/* These ones are "dual" types, might be a video, might be a parser */
-static PlaylistTypes dual_types[] = {
-	PLAYLIST_TYPE2 ("audio/x-real-audio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("audio/x-pn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("application/ram", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("application/vnd.rn-realmedia", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("audio/x-pn-realaudio-plugin", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("audio/vnd.rn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("audio/x-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("text/plain", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
-	PLAYLIST_TYPE2 ("audio/x-ms-asx", totem_pl_parser_add_asx, totem_pl_parser_is_asx),
-	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
-	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
-	PLAYLIST_TYPE2 ("video/quicktime", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
-	PLAYLIST_TYPE2 ("application/x-quicktime-media-link", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
-	PLAYLIST_TYPE2 ("application/x-quicktimeplayer", totem_pl_parser_add_quicktime, totem_pl_parser_is_quicktime),
-	PLAYLIST_TYPE2 ("application/xml", totem_pl_parser_add_xml_feed, totem_pl_parser_is_xml_feed),
-};
-
-#ifndef TOTEM_PL_PARSER_MINI
-
 static PlaylistTypes ignore_types[] = {
 	PLAYLIST_TYPE3 ("image/*"),
 	PLAYLIST_TYPE3 ("text/plain"),
@@ -1414,7 +1416,6 @@ totem_pl_parser_mimetype_is_ignored (TotemPlParser *parser,
 	}
 
 	return FALSE;
-
 }
 
 /**
@@ -1507,9 +1508,10 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url,
 		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
 	}
 
-	/* Fix up itpc, see http://www.apple.com/itunes/store/podcaststechspecs.html */
-	if (g_str_has_prefix (url, "itpc") != FALSE) {
-		DEBUG(g_print ("URL '%s' is getting special cased for ITPC parsing\n", url));
+	/* Fix up itpc, see http://www.apple.com/itunes/store/podcaststechspecs.html
+	 * as well as feed:// as used by Firefox */
+	if (g_str_has_prefix (url, "itpc") != FALSE || g_str_has_prefix (url, "feed") != FALSE) {
+		DEBUG(g_print ("URL '%s' is getting special cased for ITPC/FEED parsing\n", url));
 		return totem_pl_parser_add_itpc (parser, url, base, NULL);
 	}
 	/* Try itms Podcast references, see itunes.py in PenguinTV */
@@ -1572,6 +1574,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url,
 			if (strcmp (special_types[i].mimetype, mimetype) == 0) {
 				DEBUG(g_print ("URL '%s' is special type '%s'\n", url, mimetype));
 				if (parser->priv->disable_unsafe != FALSE && special_types[i].unsafe != FALSE) {
+					DEBUG(g_print ("URL '%s' is unsafe so was ignored\n", url));
 					g_free (mimetype);
 					g_free (data);
 					return TOTEM_PL_PARSER_RESULT_IGNORED;
@@ -1588,6 +1591,12 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url,
 				if (data == NULL) {
 					g_free (mimetype);
 					mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data, parser);
+					/* If it's _still_ a text/plain, we don't want it */
+					if (mimetype == NULL || strcmp (mimetype, "text/plain") == 0) {
+						g_free (mimetype);
+						mimetype = NULL;
+						break;
+					}
 				}
 				ret = (* dual_types[i].func) (parser, url, base, data);
 				found = TRUE;
@@ -1753,6 +1762,10 @@ totem_pl_parser_parse_duration (const char *duration, gboolean debug)
 		D(g_print ("Used 00:00 format\n"));
 		return minutes * 60 + seconds;
 	}
+	if (sscanf (duration, "%d.%d", &minutes, &seconds) == 2) {
+		D(g_print ("Used broken float format (00.00)\n"));
+		return minutes * 60 + seconds;
+	}
 	/* PLS files format */
 	if (sscanf (duration, "%d", &seconds) == 1) {
 		D(g_print ("Used PLS format\n"));
@@ -1811,6 +1824,34 @@ totem_pl_parser_parse_date (const char *date_str, gboolean debug)
 
 #endif /* !TOTEM_PL_PARSER_MINI */
 
+static char *
+my_gnome_vfs_get_mime_type_for_data (gconstpointer data, int len)
+{
+	const char *mimetype;
+
+	mimetype = gnome_vfs_get_mime_type_for_data (data, len);
+
+	if (mimetype != NULL && strcmp (mimetype, "text/plain") == 0) {
+		PlaylistIdenCallback func;
+		guint i;
+
+		func = NULL;
+
+		for (i = 0; i < G_N_ELEMENTS(dual_types); i++) {
+			const char *res;
+
+			if (func == dual_types[i].iden)
+				continue;
+			func = dual_types[i].iden;
+			res = func (data, len);
+			if (res != NULL)
+				return g_strdup (res);
+		}
+	}
+
+	return g_strdup (mimetype);
+}
+
 /**
  * totem_pl_parser_can_parse_from_data:
  * @data: the data to check for parsability
@@ -1827,13 +1868,13 @@ totem_pl_parser_can_parse_from_data (const char *data,
 				     gsize len,
 				     gboolean debug)
 {
-	const char *mimetype;
+	char *mimetype;
 	guint i;
 
 	g_return_val_if_fail (data != NULL, FALSE);
 
 	/* Bad cast! */
-	mimetype = gnome_vfs_get_mime_type_for_data ((gpointer) data, (int) len);
+	mimetype = my_gnome_vfs_get_mime_type_for_data ((gpointer) data, (int) len);
 
 	if (mimetype == NULL || strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mimetype) == 0) {
 		D(g_message ("totem_pl_parser_can_parse_from_data couldn't get mimetype"));
@@ -1851,9 +1892,9 @@ totem_pl_parser_can_parse_from_data (const char *data,
 		if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
 			D(g_message ("Should be dual type '%s', making sure now", mimetype));
 			if (dual_types[i].iden != NULL) {
-				gboolean retval = (* dual_types[i].iden) (data, len);
+				gboolean retval = ((* dual_types[i].iden) (data, len) != NULL);
 				D(g_message ("%s dual type '%s'",
-							retval ? "Is" : "Is not", mimetype));
+					     retval ? "Is" : "Is not", mimetype));
 				return retval;
 			}
 			return FALSE;
